@@ -37,20 +37,11 @@
 #ifndef HAVE_W32CE_SYSTEM
 # include <locale.h>
 #endif
-#ifdef HAVE_LANGINFO_H
-#include <langinfo.h>
-#endif
 #include <limits.h>
 #ifdef HAVE_W32CE_SYSTEM
 # include <windows.h>
 #endif
 
-#undef WITH_UTF8_CONVERSION
-#if defined FALLBACK_CURSES || defined PINENTRY_CURSES || defined PINENTRY_GTK
-# include <iconv.h>
-# define WITH_UTF8_CONVERSION 1
-#endif
-
 #include <assuan.h>
 
 #include "memory.h"
@@ -86,12 +77,6 @@ static const char *flavor_flag;
  * parser.  */
 static char *remember_display;
 
-/* Flag to remember whether a warning has been printed.  */
-#ifdef WITH_UTF8_CONVERSION
-static int lc_ctype_unknown_warning;
-#endif
-
-
 static void
 pinentry_reset (int use_defaults)
 {
@@ -255,150 +240,6 @@ pinentry_assuan_reset_handler (assuan_context_t ctx, char *line)
 
 
 
-#ifdef WITH_UTF8_CONVERSION
-char *
-pinentry_utf8_to_local (const char *lc_ctype, const char *text)
-{
-  iconv_t cd;
-  const char *input = text;
-  size_t input_len = strlen (text) + 1;
-  char *output;
-  size_t output_len;
-  char *output_buf;
-  size_t processed;
-  char *old_ctype;
-  char *target_encoding;
-
-  /* If no locale setting could be determined, simply copy the
-     string.  */
-  if (!lc_ctype)
-    {
-      if (! lc_ctype_unknown_warning)
-	{
-	  fprintf (stderr, "%s: no LC_CTYPE known - assuming UTF-8\n",
-		   this_pgmname);
-	  lc_ctype_unknown_warning = 1;
-	}
-      return strdup (text);
-    }
-
-  old_ctype = strdup (setlocale (LC_CTYPE, NULL));
-  if (!old_ctype)
-    return NULL;
-  setlocale (LC_CTYPE, lc_ctype);
-  target_encoding = nl_langinfo (CODESET);
-  if (!target_encoding)
-    target_encoding = "?";
-  setlocale (LC_CTYPE, old_ctype);
-  free (old_ctype);
-
-  /* This is overkill, but simplifies the iconv invocation greatly.  */
-  output_len = input_len * MB_LEN_MAX;
-  output_buf = output = malloc (output_len);
-  if (!output)
-    return NULL;
-
-  cd = iconv_open (target_encoding, "UTF-8");
-  if (cd == (iconv_t) -1)
-    {
-      fprintf (stderr, "%s: can't convert from UTF-8 to %s: %s\n",
-               this_pgmname, target_encoding, strerror (errno));
-      free (output_buf);
-      return NULL;
-    }
-  processed = iconv (cd, (ICONV_CONST char **)&input, &input_len,
-                     &output, &output_len);
-  iconv_close (cd);
-  if (processed == (size_t) -1 || input_len)
-    {
-      fprintf (stderr, "%s: error converting from UTF-8 to %s: %s\n",
-               this_pgmname, target_encoding, strerror (errno));
-      free (output_buf);
-      return NULL;
-    }
-  return output_buf;
-}
-#endif /*WITH_UTF8_CONVERSION*/
-
-
-/* Convert TEXT which is encoded according to LC_CTYPE to UTF-8.  With
-   SECURE set to true, use secure memory for the returned buffer.
-   Return NULL on error. */
-#ifdef WITH_UTF8_CONVERSION
-char *
-pinentry_local_to_utf8 (char *lc_ctype, char *text, int secure)
-{
-  char *old_ctype;
-  char *source_encoding;
-  iconv_t cd;
-  const char *input = text;
-  size_t input_len = strlen (text) + 1;
-  char *output;
-  size_t output_len;
-  char *output_buf;
-  size_t processed;
-
-  /* If no locale setting could be determined, simply copy the
-     string.  */
-  if (!lc_ctype)
-    {
-      if (! lc_ctype_unknown_warning)
-	{
-	  fprintf (stderr, "%s: no LC_CTYPE known - assuming UTF-8\n",
-		   this_pgmname);
-	  lc_ctype_unknown_warning = 1;
-	}
-      output_buf = secure? secmem_malloc (input_len) : malloc (input_len);
-      if (output_buf)
-        strcpy (output_buf, input);
-      return output_buf;
-    }
-
-  old_ctype = strdup (setlocale (LC_CTYPE, NULL));
-  if (!old_ctype)
-    return NULL;
-  setlocale (LC_CTYPE, lc_ctype);
-  source_encoding = nl_langinfo (CODESET);
-  setlocale (LC_CTYPE, old_ctype);
-  free (old_ctype);
-
-  /* This is overkill, but simplifies the iconv invocation greatly.  */
-  output_len = input_len * MB_LEN_MAX;
-  output_buf = output = secure? secmem_malloc (output_len):malloc (output_len);
-  if (!output)
-    return NULL;
-
-  cd = iconv_open ("UTF-8", source_encoding);
-  if (cd == (iconv_t) -1)
-    {
-      fprintf (stderr, "%s: can't convert from %s to UTF-8: %s\n",
-               this_pgmname, source_encoding? source_encoding : "?",
-               strerror (errno));
-      if (secure)
-        secmem_free (output_buf);
-      else
-        free (output_buf);
-      return NULL;
-    }
-  processed = iconv (cd, (ICONV_CONST char **)&input, &input_len,
-                     &output, &output_len);
-  iconv_close (cd);
-  if (processed == (size_t) -1 || input_len)
-    {
-      fprintf (stderr, "%s: error converting from %s to UTF-8: %s\n",
-               this_pgmname, source_encoding? source_encoding : "?",
-               strerror (errno));
-      if (secure)
-        secmem_free (output_buf);
-      else
-        free (output_buf);
-      return NULL;
-    }
-  return output_buf;
-}
-#endif /*WITH_UTF8_CONVERSION*/
-
-
 /* Copy TEXT or TEXTLEN to BUFFER and escape as required.  Return a
    pointer to the end of the new buffer.  Note that BUFFER must be
    large enough to keep the entire text; allocataing it 3 times of
@@ -464,7 +305,6 @@ get_cmdline (unsigned long pid)
   size_t i, n;
 
   snprintf (buffer, sizeof buffer, "/proc/%lu/cmdline", pid);
-  buffer[sizeof buffer - 1] = 0;
 
   fp = fopen (buffer, "rb");
   if (!fp)
@@ -510,7 +350,6 @@ get_pid_name_for_uid (unsigned long pid, int uid)
   char *uidstr;
 
   snprintf (buffer, sizeof buffer, "/proc/%lu/status", pid);
-  buffer[sizeof buffer - 1] = 0;
 
   fp = fopen (buffer, "rb");
   if (!fp)
@@ -525,6 +364,7 @@ get_pid_name_for_uid (unsigned long pid, int uid)
   fclose (fp);
   if (n == 0)
     return NULL;
+  buffer[n] = 0;
   /* Fixme: Is it specified that "Name" is always the first line?  For
    * robustness I would prefer to have a real parser here. -wk  */
   if (strncmp (buffer, "Name:\t", 6))
@@ -544,6 +384,13 @@ get_pid_name_for_uid (unsigned long pid, int uid)
 #endif /*!HAVE_W32_SYSTEM*/
 
 
+const char *
+pinentry_get_pgmname (void)
+{
+  return this_pgmname;
+}
+
+
 /* Return a malloced string with the title.  The caller mus free the
  * string.  If no title is available or the title string has an error
  * NULL is returned.  */
@@ -563,7 +410,7 @@ pinentry_get_title (pinentry_t pe)
       char *cmdline = NULL;
 
       if (pe->owner_host &&
-          !uname (&utsbuf) && utsbuf.nodename &&
+          !uname (&utsbuf) &&
           !strcmp (utsbuf.nodename, pe->owner_host))
         {
           pidname = get_pid_name_for_uid (pe->owner_pid, pe->owner_uid);
@@ -580,7 +427,6 @@ pinentry_get_title (pinentry_t pe)
       else
         snprintf (buf, sizeof buf, "[%lu] <unknown host>",
                   pe->owner_pid);
-      buf[sizeof buf - 1] = 0;
       free (pidname);
       free (cmdline);
       title = strdup (buf);
@@ -978,7 +824,6 @@ my_strusage( int level )
               {
                 snprintf (str, n, "Usage: %s [options] (-h for help)",
                           this_pgmname);
-                str[n-1] = 0;
               }
           }
         p = str;
@@ -1464,7 +1309,6 @@ write_status_error (assuan_context_t ctx, pinentry_t pe)
             pe->specific_err_loc? pe->specific_err_loc : "?",
             pe->specific_err,
             pe->specific_err_info? pe->specific_err_info : "");
-  buf[sizeof buf -1] = 0;
   assuan_write_status (ctx, "ERROR", buf);
 }
 
@@ -1883,9 +1727,11 @@ cmd_getpin (assuan_context_t ctx, char *line)
     {
       if (pinentry.repeat_okay)
         assuan_write_status (ctx, "PIN_REPEATED", "");
+      assuan_begin_confidential (ctx);
       result = assuan_send_data (ctx, pinentry.pin, strlen(pinentry.pin));
       if (!result)
 	result = assuan_send_data (ctx, NULL, 0);
+      assuan_end_confidential (ctx);
 
       if (/* GPG Agent says it's okay.  */
 	  pinentry.allow_external_password_cache && pinentry.keyinfo
@@ -2021,7 +1867,6 @@ cmd_getinfo (assuan_context_t ctx, char *line)
     {
 
       snprintf (buffer, sizeof buffer, "%lu", (unsigned long)getpid ());
-      buffer[sizeof buffer -1] = 0;
       rc = assuan_send_data (ctx, buffer, strlen (buffer));
     }
   else if (!strcmp (line, "flavor"))
@@ -2035,7 +1880,6 @@ cmd_getinfo (assuan_context_t ctx, char *line)
                 s,
                 flavor_flag? ":":"",
                 flavor_flag? flavor_flag : "");
-      buffer[sizeof buffer -1] = 0;
       rc = assuan_send_data (ctx, buffer, strlen (buffer));
       /* if (!rc) */
       /*   rc = assuan_write_status (ctx, "FEATURES", "tabbing foo bar"); */
@@ -2061,7 +1905,6 @@ cmd_getinfo (assuan_context_t ctx, char *line)
 #endif
                 emacs_status
                 );
-      buffer[sizeof buffer -1] = 0;
       rc = assuan_send_data (ctx, buffer, strlen (buffer));
     }
   else
