@@ -31,9 +31,6 @@
 # include <sys/mman.h>
 # include <sys/types.h>
 # include <fcntl.h>
-# ifdef USE_CAPABILITIES
-#  include <sys/capability.h>
-# endif
 #endif
 #include <string.h>
 
@@ -51,7 +48,7 @@ typedef union {
     short b;
     char c[1];
     long d;
-#ifdef HAVE_U64_TYPEDEF
+#ifdef HAVE_U64_TYPE
     u64 e;
 #endif
     float f;
@@ -103,7 +100,9 @@ struct memblock_struct {
 
 static void  *pool;
 static volatile int pool_okay; /* may be checked in an atexit function */
+#if HAVE_MMAP
 static int   pool_is_mmapped;
+#endif
 static size_t poolsize; /* allocated length */
 static size_t poollen;	/* used length */
 static MEMBLOCK *unused_blocks;
@@ -128,26 +127,7 @@ print_warn(void)
 static void
 lock_pool( void *p, size_t n )
 {
-#if defined(USE_CAPABILITIES) && defined(HAVE_MLOCK)
-    int err;
-
-    cap_set_proc( cap_from_text("cap_ipc_lock+ep") );
-    err = mlock( p, n );
-    if( err && errno )
-	err = errno;
-    cap_set_proc( cap_from_text("cap_ipc_lock+p") );
-
-    if( err ) {
-	if( errno != EPERM
-	  #ifdef EAGAIN  /* OpenBSD returns this */
-	    && errno != EAGAIN
-	  #endif
-	  )
-	    log_error("can't lock memory: %s\n", strerror(err));
-	show_warning = 1;
-    }
-
-#elif defined(HAVE_MLOCK)
+#if defined(HAVE_MLOCK)
     uid_t uid;
     int err;
 
@@ -156,17 +136,13 @@ lock_pool( void *p, size_t n )
 #ifdef HAVE_BROKEN_MLOCK
     if( uid ) {
 	errno = EPERM;
-	err = errno;
+	err = -1;
     }
     else {
 	err = mlock( p, n );
-	if( err && errno )
-	    err = errno;
     }
 #else
     err = mlock( p, n );
-    if( err && errno )
-	err = errno;
 #endif
 
     if( uid && !geteuid() ) {
@@ -180,11 +156,13 @@ lock_pool( void *p, size_t n )
 	    && errno != EAGAIN
 #endif
 	  )
-	    log_error("can't lock memory: %s\n", strerror(err));
+	    log_error("can't lock memory: %s\n", strerror(errno));
 	show_warning = 1;
     }
 
 #else
+    (void)p;
+    (void)n;
     log_info("Please note that you don't have secure memory on this system\n");
 #endif
 }
@@ -193,20 +171,22 @@ lock_pool( void *p, size_t n )
 static void
 init_pool( size_t n)
 {
+#if HAVE_MMAP
     size_t pgsize;
+#endif
 
     poolsize = n;
 
     if( disable_secmem )
 	log_bug("secure memory is disabled");
 
+#if HAVE_MMAP
 #ifdef HAVE_GETPAGESIZE
     pgsize = getpagesize();
 #else
     pgsize = 4096;
 #endif
 
-#if HAVE_MMAP
     poolsize = (poolsize + pgsize -1 ) & ~(pgsize-1);
 # ifdef MAP_ANONYMOUS
        pool = mmap( 0, poolsize, PROT_READ|PROT_WRITE,
@@ -284,11 +264,7 @@ void
 secmem_init( size_t n )
 {
     if( !n ) {
-#ifdef USE_CAPABILITIES
-	/* drop all capabilities */
-	cap_set_proc( cap_from_text("all-eip") );
-
-#elif !defined(HAVE_DOSISH_SYSTEM)
+#if !defined(HAVE_DOSISH_SYSTEM)
 	uid_t uid;
 
 	disable_secmem=1;
